<?php
/*
  jsi_functions.php
  General purpose functions used in many places.
  Standard (template) edition
  TLD 2008-07-10
  TLD 2008-10-11 Add GetVarDump
  TLD 2009-02-12 Add DoubleQuoteString
  TLD 2009-06-29 Add FormatPhone
  TLD 2009-07-09 Replace ereg (deprecated) with preg_match
  TLD 2009-09-25 Copy image unchanged if not in cache folder and doesn't need resizing
  TLD 2009-11-03 Add DoubleQuoteStringComma
  TLD 2010-01-28 Convert values in QuoteString functions to strings
  TLD 2010-02-04 Add new param to thumbimage for new image filename (large/small names)
  TLD 2010-09-13 Add new function RemoveQuotes.
  TLD 2010.09.23 MySQL can't handle double quotes around table/column names
*/

function GetParam($pParamName) {
  $reqVal = @$_REQUEST[$pParamName];
  if ($reqVal > '') return $reqVal;
  return '';
}

function WriteToFile($filename, $mode, $content, $newline = TRUE) {
  # mode is 'w' (write) or 'a' (append)
  # write mode will overwrite any existing content in a file
  # both modes will try to create a new file if it does not exist
  # always append a newline unless the optional 4th param is FALSE
  if (file_exists($filename) && !is_writable($filename)) {
     if (!chmod($filename, 0666)) {
        GenericWarn("Cannot change mode to writable for file: $filename");
        return false;
     };
  }
  if (!$fp = @fopen($filename, $mode)) {
     GenericWarn("Cannot open file: $filename");
     return false;
  }
  if ($newline === TRUE) {
     if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
       $content .= "\r";
     $content .= "\n";
  }
  if (fwrite($fp, $content) === FALSE) {
     GenericWarn("Cannot write to file: $filename");
     return false;
  }
  if (!fclose($fp)) {
     GenericWarn("Cannot close file: $filename");
     return false;
  }
  return true;
}

function WriteLog($msg) {
  $ok = false;
  if (@constant("JSI_SYNC_LOG") > " ") {
     $ok = WriteToFile(JSI_SYNC_LOG, "a", $msg);
     if (!$ok) GenericWarn("Could not write to sync log file");
  }
  return $ok;
}

function DebugLog($msg) {
  // note the @ suppresses any error messages if constant is not defined
  if (@constant("JSI_DEBUG_MODE") === TRUE) {
    WriteLog('--DEBUG: ' . $msg);
  }
}

function BeginLogEntry() {
  $msg = "***** Begin Request " . date(defined("DATE_ATOM")?DATE_ATOM:"r") . " ***";
  WriteLog($msg);
  if (CLI_CLIENT) {
    WriteLog('Addr: localhost');
    WriteLog('Type: Command Line Interface (CLI)');
  } else {
    WriteLog('Addr: ' . @$_SERVER["REMOTE_ADDR"]);
    WriteLog('Type: ' . @$_SERVER["HTTP_USER_AGENT"]);
  }
  foreach ($_REQUEST as $key => $value) {
    DebugLog("Request parameter: $key=$value");
  }
}

function EndLogEntry() {
  $msg = "***** Ended Request " . date(defined("DATE_ATOM")?DATE_ATOM:"r") . " ***";
  WriteLog($msg);
  WriteLog('');
}

function MsgAndLog($msg) {
  WriteLog($msg);
  GenericMsg($msg);
}
function GenericMsg($msg) {
  // if request is from Jargon Reader, output jargon CIO message box
  // if request is from a browser, output HTML
  // else output msg text only (CLI mode)
  if (JARGON_CLIENT)
    jsiMsg($msg);
  else
  if (BROWSER_CLIENT)
    echo $msg . "<BR>";
  else 
    echo $msg;
}
function WarnAndLog($msg) {
  WriteLog("**WARNING** $msg");
  GenericWarn($msg);
}
function GenericWarn($msg) {
  // if request is from Jargon Reader, output jargon CIO message box
  // if request is from a browser, output HTML
  // else output msg text only (CLI mode)
  if (JARGON_CLIENT)
    jsiWarn($msg);
  else
  if (BROWSER_CLIENT)
    echo $msg . "<BR>";
  else 
    echo $msg;
}

// return the formatted output of var_dump in a string value
function GetVarDump($mixed = null) {
  ob_start();
  var_dump($mixed);
  $content = ob_get_contents();
  ob_end_clean();
  return $content;
}

function RemoveQuotes($val, $do_trim) {
  if ($val == null || $val == '')
    return '';
  if (is_array($val)) {
    // remove double quotes around each array value and trim it
    for ($ix=0; $ix<count($val); $ix++) {
      $val[$ix] = (string)$val[$ix];
      $val[$ix] = trim($val[$ix], '"');  // strip leading/trailing double quotes
      if ($do_trim)
        $val[$ix] = trim($val[$ix]);     // optionally trim whitespace chars
    }
  }
  else {
    // just a character string
    $val = (string)$val;
    $val = trim($val, '"');  // strip leading/trailing double quotes
    if ($do_trim)
      $val = trim($val);     // optionally trim whitespace chars
  }
  return $val;
}
function DoubleQuoteString($str_val) {
  if ($str_val == null)
    $str_val = '';
  else
    $str_val = (string)$str_val;  // TLD 2010-01-28
  if ( strpos($str_val, ";") !== false )
    $str_val = str_replace(";", " ", $str_val);
  // TLD 2010.09.23 MySQL can't handle double quotes around table/column names
  // return ('"' . str_replace('"', '""', $str_val) . '"');
  return $str_val;
}
function DoubleQuoteStringComma($str_val) {
  return (DoubleQuoteString($str_val) . ", ");
}
function QuoteString($str_val) {
  if ($str_val == null)
    $str_val = '';
  else
    $str_val = (string)$str_val;  // TLD 2010-01-28
  if ( strpos($str_val, ";") !== false )
    $str_val = str_replace(";", " ", $str_val);
  return ("'" . str_replace("'", "''", $str_val) . "'");
}
function QuoteStringComma($str_val) {
  return (QuoteString($str_val) . ", ");
}

// makes sure a directory pathname ends with a slash
function FormatDir($dir) {
  if (EndsWith($dir, '/'))
    return $dir;
  else
    return $dir . '/';
}

// format telephone number as 999.999.9999 x9999 or substring thereof
function FormatPhone($phone_num, $sep='.') {
  if ( $phone_num == '0000000' || $phone_num == '0000000000' )
    return '';
  $len = strlen($phone_num);
  if (   $phone_num == ''  
      || $len < 7
      || strpos($phone_num, '(') !== false  // fudge matching )
      || strpos($phone_num, '-') !== false
      || strpos($phone_num, '.') !== false
      || strpos($phone_num, ' ') !== false )
    return $phone_num;
  if ($len == 7)
    return substr($phone_num, 0, 3) 
         . $sep 
         . substr($phone_num, 3, 4);
  if ($len > 10 && substr($phone_num, 0, 1) == '1') {
    $phone_num = substr($phone_num, 1);
    $len--;
  }
  if ($len == 10)
    return substr($phone_num, 0, 3)
         . $sep
         . substr($phone_num, 3, 3)
         . $sep
         . substr($phone_num, 6, 4);
  if ($len > 10)
    return substr($phone_num, 0, 3)
         . $sep
         . substr($phone_num, 3, 3)
         . $sep
         . substr($phone_num, 6, 4)
         . ' x'
         . substr($phone_num, 10);
  return $phone_num;
}

// returns true if $str begins with $sub
function BeginsWith($str, $sub) {
  if (strlen($str) < strlen($sub)) return false;
  return (strncmp($str, $sub, strlen($sub)) == 0);
}

// returns true if $str ends with $sub
function EndsWith($str, $sub) {
  if (strlen($str) < strlen($sub)) return false;
  $tail = substr($str, strlen($str) - strlen($sub));
  return (strncmp($tail, $sub, strlen($sub)) == 0);
}

// if $off is a number, trim "n" chars from the front of $str
// otherwise trim the matching string in $off from the front of $str
function TrimOffFront($off, $str) {
  if (is_numeric($off))
    return substr($str, $off);
  else
    return substr($str, strlen($off));
}

// if $off is a number, trim "n" chars from the end of $str
// otherwise trim the matching string in $off from the end of $str
function TrimOffEnd($off, $str) {
  if(is_numeric($off))
    return substr($str, 0, strlen($str) - $off);
  else
    return substr($str, 0, strlen($str) - strlen($off));
}

/*
  Utility class: static methods for cleaning & escaping untrusted (i.e.
  user-supplied) strings.

  Any string can (usually) be thought of as being in one of these 'modes':

  pure = what the user actually typed / what you want to see on the page /
         what is actually stored in the DB
  gpc  = incoming GET, POST or COOKIE data
  sql  = escaped for passing safely to RDBMS via SQL (also, data from DB
         queries and file reads if you have magic_quotes_runtime on--which
         is rare)
  html = safe for html display (htmlentities applied)

  Always knowing what mode your string is in--using these methods to
  convert between modes--will prevent SQL injection and cross-site scripting.

  This class refers to its own namespace (so it can work in PHP 4--there is no
  self keyword until PHP 5). Do not change the name of the class w/o changing
  all the internal references.

  Example usage: a POST value that you want to query with:
  $username = Str::gpc2sql($_POST['username']);
*/

// This sets SQL escaping to use slashes;
// for Sybase(/MSSQL)-style escaping ( ' --> '' ), set to true. '
define('STR_SYBASE', false);

class Str {
  function gpc2sql($gpc, $maxLength = false) {
    return Str::pure2sql(Str::gpc2pure($gpc), $maxLength);
  }
  function gpc2html($gpc, $maxLength = false) {
    return Str::pure2html(Str::gpc2pure($gpc), $maxLength);
  }
  function gpc2pure($gpc) {
    if (ini_get('magic_quotes_sybase'))
      $pure = str_replace("''", "'", $gpc);
    else $pure = get_magic_quotes_gpc() ? stripslashes($gpc) : $gpc;
      return $pure;
  }
  function html2pure($html) {
    return html_entity_decode($html);
  }
  function html2sql($html, $maxLength = false) {
    return Str::pure2sql(Str::html2pure($html), $maxLength);
  }
  function pure2html($pure, $maxLength = false) {
    return $maxLength
           ? htmlentities(substr($pure, 0, $maxLength))
           : htmlentities($pure);
  }
  function pure2sql($pure, $maxLength = false) {
    if ($maxLength) $pure = substr($pure, 0, $maxLength);
    return (STR_SYBASE)
           ? str_replace("'", "''", $pure)
           : addslashes($pure);
  }
  function sql2html($sql, $maxLength = false) {
    $pure = Str::sql2pure($sql);
    if ($maxLength) $pure = substr($pure, 0, $maxLength);
    return Str::pure2html($pure);
  }
  function sql2pure($sql) {
    return (STR_SYBASE)
           ? str_replace("''", "'", $sql)
           : stripslashes($sql);
  }
}

// create a JPG thumbnail image from another image in any of various image formats
// TLD 2009-07-24 add new param to this fn call
function thumbimage ($image, $x, $y, $aspectratio, $resize, $cachedir, $newbasename){
  if (!defined('JPEGQUALITY'))
     define('JPEGQUALITY', 95);        //define the quality of JPG thumbnails

  // error_reporting(0);
  error_reporting(E_ALL);

  $types = array (1 => "gif", "jpeg", "png", "swf", "psd", "wbmp");
  // Formats Not Supported - To use this feature uncomment the line below
  // and add the files types you do not want thumbed.

  // TLD 20070829 remove GIF from list of unsupported image types
  // $not_supported_formats = array ("GIF");
  $not_supported_formats = array ("SWF");

  umask(0);
  !is_dir ($cachedir)
     ? mkdir ($cachedir, 0777)
     : chmod ($cachedir, 0777);

  // at least one of x or y must be set, and if set, must be an integer
  if (! ( (!isset ($x) || preg_match ('/^[0-9]{1,}$/', $x, $regs)) &&
          (!isset ($y) || preg_match ('/^[0-9]{1,}$/', $y, $regs)) &&
          ( isset ($x) || isset ($y))
        )
     ) {
     WarnAndLog(__FUNCTION__ . ": Missing or invalid size parameter(s)");
     return false;
  }

  !isset ($resize) || !preg_match ('/^[0|1]$/', $resize, $regs)
     ? $resize = 0
     : $resize;

  !isset ($aspectratio) || !preg_match ('/^[0|1]$/', $aspectratio, $regs)
     ? isset ($x) && isset ($y)
          ? $aspectratio = 1
          : $aspectratio = 0
     : $aspectratio;

  if (!isset ($image)) {
     WarnAndLog(__FUNCTION__ . ": No image was specified");
     return false;
  }
  if (!file_exists ($image)) {
     WarnAndLog(__FUNCTION__ . ": Specified image file not found on server: $image");
     return false;
  }

  $imagedata = getimagesize($image);

  if (!$imagedata[2] || $imagedata[2] == 4 || $imagedata[2] == 5) {
     WarnAndLog(__FUNCTION__ . ": Specified image file is not a supported type: " . basename($image));
     return false;
  }

  // TLD 2009-07-24 add new param to this fn call
  // $thumbfile =  "/" . basename($image);
  $thumbfile =  "/" . $newbasename;

  $imgtype="!(ImageTypes() & IMG_" . strtoupper($types[$imagedata[2]]) . ");";
  if ((eval($imgtype)) ||
      (in_array(strtoupper(array_pop(explode(".", basename($image)))),$not_supported_formats))) {
     copy($image, $cachedir.$thumbfile);
     WriteLog("Could not create thumbnail, image copied unchanged: " . basename($image));
     $image = $cachedir.$thumbfile;
     return $image;
  }

  if (!isset ($x)) $x = floor ($y * $imagedata[0] / $imagedata[1]);
  if (!isset ($y)) $y = floor ($x * $imagedata[1] / $imagedata[0]);
  if ($aspectratio && isset ($x) && isset ($y)) {
     if ( ($imagedata[1]/$y) > ($imagedata[0]/$x) ) {
        $x=ceil(($imagedata[0]/$imagedata[1])* $y);
     } else {
        $y=ceil($x/($imagedata[0]/$imagedata[1]));
     }
  }

  if (file_exists ($cachedir.$thumbfile)) {
     $thumbdata = getimagesize ($cachedir.$thumbfile);
     $thumbdata[0] == $x && $thumbdata[1] == $y
        ? $iscached = true
        : $iscached = false;
     if (@filemtime($cachedir.$thumbfile) < @filemtime($image))
        $iscached = false;
  } else {
     $iscached = false;
  }

  if (!$iscached) {
     if ( ($imagedata[0] > $x || $imagedata[1] > $y) ||
         (($imagedata[0] < $x || $imagedata[1] < $y) && $resize) )
       $makethumb = true;
     else {
       $makethumb = false;
       // 2009-09-25 copy unchanged if not in cache folder and doesn't need resizing
       copy($image, $cachedir.$thumbfile);
     }
  } else {
     $makethumb = false;
  }

  if ($makethumb) {
     $image = call_user_func("imagecreatefrom".$types[$imagedata[2]], $image);
     if (function_exists("imagecreatetruecolor") && ($thumb = imagecreatetruecolor ($x, $y))) {
        imagecopyresampled ($thumb, $image, 0, 0, 0, 0, $x, $y, $imagedata[0], $imagedata[1]);
     } else {
        $thumb = imagecreate ($x, $y);
        imagecopyresized ($thumb, $image, 0, 0, 0, 0, $x, $y, $imagedata[0], $imagedata[1]);
     }
     if ($types[$imagedata[2]] == "jpeg") {
        call_user_func("image".$types[$imagedata[2]], $thumb, $cachedir.$thumbfile, JPEGQUALITY);
     }
     else {
        call_user_func("image".$types[$imagedata[2]], $thumb, $cachedir.$thumbfile);
     }
     imagedestroy ($image);
     imagedestroy ($thumb);
     $image = $cachedir.$thumbfile;
  } else {
     if ($iscached)
        $image = $cachedir.$thumbfile;
  }
  return $image;
}

// This function makes a new password from a plaintext password.
  function jsi_encrypt_password($plain) {
    $password = "";
    for ($i=0; $i<10; $i++) {
      $password .= jsi_rand();
    }
    $salt = substr(md5($password), 0, 2);
    $password = md5($salt . $plain) . ":" . $salt;
    return $password;
  }

// Return a random number
  function jsi_rand($min = null, $max = null) {
    static $seeded;
    if (!isset($seeded)) {
      mt_srand((double)microtime()*1000000);
      $seeded = true;
    }
    if (isset($min) && isset($max)) {
      if ($min >= $max)
        return $min;
      else
        return mt_rand($min, $max);
    }
    else
      return mt_rand();
  }

// return a random string of length $max
  function jsi_random_string($max = 20){
    $chars = "abcdefghijklmnopqrstuvwxwz0123456789_";
    for($i = 0; $i < $max; $i++){
      $rand_key = mt_rand(0, strlen($chars));
      $string  .= substr($chars, $rand_key, 1);
    }
    return str_shuffle($string);
  }

?>
